#!/usr/bin/env python

"""
Classes encapsulating decypher tool.
"""
# import galaxy
# from galaxy import eggs
# from galaxy import datatypes
# from galaxy.datatypes import velvet
# from galaxy.datatypes.velvet import Velvet
import pkg_resources
import logging, os, string, sys, tempfile, glob, shutil, types, urllib
import shlex, subprocess
from optparse import OptionParser, OptionGroup
from stat import *


log = logging.getLogger( __name__ )

assert sys.version_info[:2] >= ( 2, 4 )

def visitfile(file):
    print >> sys.stderr,  'visiting', file , os.stat(file)[ST_SIZE], os.stat(file)[ST_CTIME]

def walktree(top, callback=visitfile):
    '''recursively descend the directory tree rooted at top,
       calling the callback function for each regular file'''

    for f in os.listdir(top):
        pathname = os.path.join(top, f)
        mode = os.stat(pathname)[ST_MODE]
        if S_ISDIR(mode):
            # It's a directory, recurse into it
            walktree(pathname, callback)
        elif S_ISREG(mode):
            # It's a file, call the callback function
            callback(pathname)
        else:
            # Unknown file type, print a message
            print 'Skipping %s' % pathname

def exec_after_process(app, inp_data, out_data, param_dict, tool=None, stdout=None, stderr=None):
    items = out_data.items()
    print >> sys.stderr, 'JJ items ', items  # so will appear as blurb for file

def stop_err( msg ):
    sys.stderr.write( "%s\n" % msg )
    sys.exit()

def __main__():
    #Parse Command Line
    s = 'velveth_wrapper.py:  argv = %s\n' % (sys.argv)
    # print >> sys.stderr, s # so will appear as blurb for file
    argcnt = len(sys.argv)
    html_file = sys.argv[1]
    working_dir = sys.argv[2]
    """
    try: # for test - needs this done
        os.mkdir(working_dir)
    except Exception, e:
        stop_err( 'Error running velveth ' + str( e ) )
    """
    try: # for test - needs this done
        os.makedirs(working_dir)
    except Exception, e:
        stop_err( 'Error running velveth ' + str( e ) )
    walktree(working_dir,visitfile)
    hash_length = sys.argv[3]
    inputs = string.join(sys.argv[4:],' ')
    cmdline = 'velveth %s %s %s > /dev/null' % (working_dir, hash_length, inputs)
    # print >> sys.stderr, cmdline # so will appear as blurb for file
    try:
        proc = subprocess.Popen( args=cmdline, shell=True, stderr=subprocess.PIPE )
        returncode = proc.wait()
        # get stderr, allowing for case where it's very large
        stderr = ''
        buffsize = 1048576
        try:
            while True:
                stderr += proc.stderr.read( buffsize )
                if not stderr or len( stderr ) % buffsize != 0:
                    break
        except OverflowError:
            pass
        if returncode != 0:
            raise Exception, stderr
    except Exception, e:
        stop_err( 'Error running velveth ' + str( e ) )
    # print >> sys.stderr, 'Done ' + cmdline # so will appear as blurb for file
    # walktree(working_dir,visitfile)
    log_path = os.path.join(working_dir,'Log')
    # print >> sys.stderr, 'JJ after walktree ', log_path  # so will appear as blurb for file
    sequences_path = os.path.join(working_dir,'Sequences')
    roadmaps_path = os.path.join(working_dir,'Roadmaps')
    # print >> sys.stderr, 'JJ 3 ', log_path  # so will appear as blurb for file
    # velvet = galaxy.datatypes.velvet.Velvet()
    # velvet.set_meta(html_file)
    rval = ['<html><head><title>Velvet Galaxy Composite Dataset </title></head><p/>']
    rval.append('<div>%s<p/></div>' % (cmdline) )
    rval.append('<div>This composite dataset is composed of the following files:<p/><ul>')
    rval.append( '<li><a href="%s" type="text/plain">%s </a>%s</li>' % (log_path,'Log','Log' ) )
    rval.append( '<li><a href="%s" type="text/plain">%s </a>%s</li>' % (sequences_path,'Sequences','Sequences' ) )
    rval.append( '<li><a href="%s" type="text/plain">%s </a>%s</li>' % (roadmaps_path,'Roadmaps','Roadmaps' ) )
    rval.append( '</ul></div></html>' )
    f = file(html_file,'w')
    f.write("\n".join( rval ))
    f.write('\n')
    f.close()
    # out = open(html_file,'w')
    # print >> sys.stderr, 'JJ 4 opened ', html_file  # so will appear as blurb for file
    # for line in open( log_path ):
    #     out.write( "%s" % (line) )
    # print >> sys.stderr, 'JJ 5 closing ', html_file  # so will appear as blurb for file
    # out.write( "%s" % (sequences_path) )
    # out.write( "%s" % (roadmaps_path) )
    # out.close()

if __name__ == "__main__": __main__()
